@@ -40,7 +40,12 @@ def diff(a,b):
 	for x in a:
 		if b.get(x)!=a[x]: diff[x]=[a[x],b.get(x)]
 	return diff
-def notice(x,c='default'):
+def notice(x,c='default',silent=None):
+	if silent:
+		red=redis()
+		if red.exists(x):return
+		else:
+			red.set(x,'1',silent)
 	try:request.urlopen(r'http://call.ff2.pw/?%s=%s&passwd=frank'%(c,urllib.parse.quote_plus(x)))
 	except:pass
 def rand_sleep(i=0.4,a=0.8):import random;time.sleep(random.randint(i*1000,a*1000)/1000);
@@ -117,14 +122,20 @@ def decrypt(string,password=None):
 def start():
 	global start_time
 	start_time=now('micro')
-def end(is_print=True):
-	if is_print : print(now('micro')-start_time)
-	else: return now('micro')-start_time
+def end(_type=True):#True:print False:return number:sleep(number)
+	if _type is True : print(now('micro')-start_time)
+	elif _type is False:return now('micro')-start_time
+	elif isinstance(_type,(int,float)):
+		_=_type-(now('micro')-start_time)
+		_>0 and time.sleep(_)
 class mysql(object):
 	conn=None
-	def __init__(self, host=None,port=3306,**args):
+	def __init__(self, host=None,port=3306,threadsafe=False,**args):
 		import pymysql
 		self.pymysql=pymysql
+		if threadsafe:
+			self.lock=threading.Lock()
+		else:self.lock=None
 		if args.get('ssh_host'):
 			import sshtunnel
 			sshtunnel.DAEMON=True
@@ -133,7 +144,7 @@ class mysql(object):
 			self.server = sshtunnel.SSHTunnelForwarder(ssh_address_or_host=args.pop('ssh_host'),ssh_username=args.pop('ssh_username'),ssh_password=args.pop('ssh_password'),remote_bind_address=('127.0.0.1' if host is None else host,port))
 			self.server.start()
 			port=self.server.local_bind_port
-		connect_config={'host':'127.0.0.1' if host is None else host,'port':port,'user':'root','password':'','db':'localhost','charset':'utf8','cursorclass':pymysql.cursors.DictCursor,'autocommit':None}
+		connect_config={'host':'127.0.0.1' if host is None else host,'port':port,'user':'root','password':'','db':'localhost','cursorclass':pymysql.cursors.DictCursor,'autocommit':None}
 		if 'table' in args:
 			self.table=args['table']
 			del args['table']
@@ -187,17 +198,28 @@ class mysql(object):
 	def execute(self,strs):
 		if not self.conn:self.__init_connect__()
 		strs=strs.strip()
-		is_query = True if strs[:4].lower() in ['sele','show'] else False
+		op_type=strs[:4].lower()
+		is_query = True if op_type in ['sele','show','desc'] else False
 		for x in range(3):
 			try:
+				if self.lock:self.lock.acquire(10)
 				with self.conn.cursor() as cur:
 					cur.execute(strs)
-					data=cur.fetchall() if is_query else cur.lastrowid
+					if is_query:
+						data=cur.fetchall()
+					elif op_type in ['upda','dele']:
+						data=cur.rowcount
+					else:
+						data=cur.lastrowid
 				return data
 			except Exception as e:
-				if '2006' in str(e) or '2013' in str(e):
-					self.__init_connect__()
+				if '2006' in str(e) or '2013' in str(e) or isinstance(e,self.pymysql.err.InterfaceError):
+					if self.conn:
+						self.conn.close();self.conn=None;
+						self.__init_connect__()
 				else:raise Exception(e)
+			finally:
+				if self.lock:self.lock.release()
 		# is_query or self.conn.commit()
 		raise Exception('MYSQL:try 3 times connect,but error')
 	def add(self,dicts,table=None,ignore=False,duplicate=[]):
@@ -228,7 +250,10 @@ class mysql(object):
 		table=self.table if table is None else table
 		where=self._split_dict(where,'and') if isinstance(where,dict) else where
 		data=self._split_dict(data)
-		return self.execute(r''' update `%s` set %s where %s'''%(table,data,where))
+		return self.execute(r' update `%s` set %s where %s'%(table,data,where))
+	def delete(self,where,table=None,where_splitor='and'):
+		where=self._split_dict(where,where_splitor) if isinstance(where,dict) else where
+		return self.execute(r' delete from `%s` where %s'%(table,where))
 	def select(self,where=None,table=None,where_splitor='and',field='*',order=None,limit=None):
 		table=self.table if table is None else table
 		query_str=f'select {field} from {table} '
@@ -240,7 +265,10 @@ class mysql(object):
 		if limit:
 			query_str+=f'limit {limit} '
 		return self.execute(query_str)
-
+	def find(self,*args,**argv):
+		if 'order' not in argv : argv['order']='id desc'
+		res=self.select(*args,**argv,limit=1)
+		return res and (res[0][list(res[0].keys())[0]] if len(res[0])==1 else res[0])
 class redis(object):
 	server=False
 	def __init__(self,host='127.0.0.1',db=0,port=6379,ssh_host=None,ssh_username='root',ssh_password=None):
@@ -262,10 +290,10 @@ def process_arg_construct(urls:list,args):#args must be iterable
 	for i,url in enumerate(urls):
 		proc_args.append((url,args[i*every_proc_count:i*every_proc_count+every_proc_count]))
 	return proc_args
-def process(target,is_thread=True,join=False,num=1,sleep=None,**args):
-	if isinstance(target,str):#args:target,[sync=False]
+def process(target,is_thread=True,join=False,num=1,sleep=None,daemon=False,**args):
+	if isinstance(target,(str,list)):#args:target,[sync=True]
 		import subprocess
-		proc=subprocess.Popen(target,stdin=subprocess.PIPE,stdout=subprocess.PIPE,stderr=subprocess.PIPE,shell=True);
+		proc=subprocess.Popen(target,stdin=subprocess.PIPE,stdout=subprocess.PIPE,stderr=subprocess.PIPE,shell=True if isinstance(target,str) else False);
 		return proc.communicate() if args.get('sync',True) else proc
 	pros=[]
 	if 'args' in args:#args can be list or tuple : [('',),('',)]
@@ -278,6 +306,7 @@ def process(target,is_thread=True,join=False,num=1,sleep=None,**args):
 			if 'args' in args:
 				args['args']=args_list[x]
 			pros.append(threading.Thread(target=target,**args))
+			if daemon : pros[-1].daemon=True
 			pros[-1].start()
 			if sleep:time.sleep(sleep)
 	else:
@@ -286,9 +315,25 @@ def process(target,is_thread=True,join=False,num=1,sleep=None,**args):
 			if 'args' in args:
 				args['args']=args_list[x] if isinstance(args_list[x],(tuple,list)) else [args_list[x]]
 			pros.append(Process(target=target,**args))
+			if daemon : pros[-1].daemon=True
 			pros[-1].start()
 			if sleep:time.sleep(sleep)
 	if join:
 		for x in range(num):
-			pros[x].join()
-	return pros[0] if len(pros)==1 else pros
\ No newline at end of file
+			pros[x].join(None if join is True else join)
+	return pros[0] if len(pros)==1 else pros
+class ErrorCounter:
+	def __init__(self,time_interval=60):
+		from collections import deque
+		self.error_timestamps = deque()
+		self.time_interval=time_interval
+	def add(self):
+		current_time = time.time()
+		self.error_timestamps.append(current_time)
+	def count(self):
+		current_time = time.time()
+		while self.error_timestamps and self.error_timestamps[0] < current_time - self.time_interval:
+			self.error_timestamps.popleft()
+		return len(self.error_timestamps)
+	def clear(self):
+		self.error_timestamps.clear()
\ No newline at end of file
